"use strict";
/*
    Implementation of a very simple HTTP proxy using Express and Axios.
    This proxy supports optional transformations of the request before it goes to
    the target service and/or the response after it returns from the target service.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.proxy = void 0;
const axios_1 = __importDefault(require("axios"));
const express_1 = __importDefault(require("express"));
/**
 * Implements the proxy server.
 *
 * @param prxPort The port on which the proxy listens.
 * @param baseSvcUrl The URL of the target service that is being proxied.
 * @param options Optional functions to transform the request body and/or response data.
 *  The transform functions do not need to set the headers for content-type, host, and
 *  content-length as they are set automatically. If the `headers` field of a transform
 *  function response is undefined then the original untransformed headers are used, with
 *  the appropriate automatic setting of the host and content-length headers.
 */
function proxy(prxPort, baseSvcUrl, options) {
    const reqTransform = options?.reqTransform;
    const resTransform = options?.resTransform;
    const app = (0, express_1.default)();
    // Enable Express JSON middleware.
    app.use(express_1.default.json({
        verify: (req, res, buf) => {
            try {
                JSON.parse(buf.toString());
            }
            catch (e) {
                res.status(400);
                res.send("invalidJsonError");
                throw Error("Invalid JSON");
            }
        },
    }));
    app.all("/*", (reqP, resP) => {
        const method = reqP.method;
        if (method != "POST" && method != "PUT" && method != "PATCH") {
            throw new Error(`Invalid HTTP method ${reqP.method}`);
        }
        const path = reqP.path;
        const reqTransformIn = {
            path: reqP.path,
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            data: reqP.body,
            headers: reqP.headers
        };
        const reqTransformOut = reqTransform ? reqTransform(reqTransformIn) : reqTransformIn;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        const reqData = reqTransformOut.data ? reqTransformOut.data : reqTransformIn.data;
        const reqHeaders = reqTransformOut.headers ? reqTransformOut.headers : reqTransformIn.headers;
        // Let Axios generate appropriate host and content-length headers.
        delete reqHeaders.host;
        delete reqHeaders["content-length"];
        (0, axios_1.default)({
            method,
            url: baseSvcUrl + path,
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            data: reqData,
            headers: reqHeaders
        })
            .then((resS) => {
            // eslint-disable-next-line no-param-reassign
            resP.charset = "utf8";
            resP.status(resS.status);
            const resTransformIn = {
                path: reqP.path,
                status: resS.status,
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                data: resS.data,
                headers: resS.headers
            };
            const resTransformOut = resTransform ? resTransform(resTransformIn) : resTransformIn;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            const resData = resTransformOut.data ? resTransformOut.data : resTransformIn.data;
            const resHeaders = resTransformOut.headers ? resTransformOut.headers : resTransformIn.headers;
            // Let Express generate appropriate content-length header.
            delete resHeaders["content-length"];
            // eslint-disable-next-line promise/always-return
            for (const k in resHeaders) {
                const header = resHeaders[k];
                if (header) {
                    resP.setHeader(k, header);
                }
            }
            resP.send(resData);
        })
            .catch((error) => {
            resP.status(500);
            resP.send(error);
            console.error("ERROR:", error);
        });
    });
    app.listen(prxPort, () => {
        console.log(`Application is running on port ${prxPort}`);
    });
}
exports.proxy = proxy;
